@@ -56,6 +56,8 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <time.h>
+#include <sys/time.h>
 #include <netdb.h>
 #include <pwd.h>
 #include <grp.h>
@@ -65,7 +67,6 @@
 #include <sys/param.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <time.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <syslog.h>
@@ -91,6 +92,7 @@
 #include "bookkeeper.h"
 #include "nfxstat.h"
 #include "collector.h"
+#include "exporter.h"
 #include "netflow_v1.h"
 #include "netflow_v5_v7.h"
 #include "netflow_v9.h"
@@ -427,10 +429,10 @@ srecord_t	*commbuff;
 	 * for proper cleanup 
 	 */
 	while ( 1 ) {
+		struct timeval tv;
 
 		/* read next bunch of data into beginn of input buffer */
 		if ( !done) {
-
 #ifdef PCAP
 			// Debug code to read from pcap file, or from socket 
 			cnt = receive_packet(socket, in_buff, NETWORK_INPUT_BUFF_SIZE , 0, 
@@ -459,7 +461,10 @@ srecord_t	*commbuff;
 		}
 
 		/* Periodic file renaming, if time limit reached or if we are done.  */
-		t_now = time(NULL);
+		// t_now = time(NULL);
+		gettimeofday(&tv, NULL);
+		t_now = tv.tv_sec;
+
 		if ( ((t_now - t_start) >= twin) || done ) {
 			char subfilename[64];
 			struct  tm *now;
@@ -534,6 +539,8 @@ srecord_t	*commbuff;
 					ResetBppHistogram(fs->xstat->bpp_histogram);
 				}
 
+				// Flush Exporter Stat to file
+				FlushExporterStats(fs);
 				// Close file
 				CloseUpdateFile(nffile, fs->Ident);
 
@@ -582,12 +589,12 @@ srecord_t	*commbuff;
 					}
 					/* XXX needs fixing */
 					if ( fs->xstat ) {
-						SetFlag(nffile->file_header->flags, FLAG_EXTENDED_STATS);
+						// to be implemented
 					}
 				}
 
-				// Dump all extension maps to the buffer
-				FlushExtensionMaps(fs);
+				// Dump all extension maps and exporters to the buffer
+				FlushStdRecords(fs);
 
 				// next flow source
 				fs = fs->next;
@@ -658,9 +665,22 @@ srecord_t	*commbuff;
 		// get flow source record for current packet, identified by sender IP address
 		fs = GetFlowSource(&nf_sender);
 		if ( fs == NULL ) {
-			syslog(LOG_WARNING, "Skip UDP packet. Ignored packets so far %u packets", ignored_packets);
-			ignored_packets++;
-			continue;
+			fs = AddDynamicSource(&FlowSource, &nf_sender);
+			if ( fs == NULL ) {
+				syslog(LOG_WARNING, "Skip UDP packet. Ignored packets so far %u packets", ignored_packets);
+				ignored_packets++;
+				continue;
+			}
+			if ( InitBookkeeper(&fs->bookkeeper, fs->datadir, getpid(), launcher_pid) != BOOKKEEPER_OK ) {
+				LogError("Failed to initialise bookkeeper for new source");
+				// fatal error
+				return;
+			}
+			fs->nffile = OpenNewFile(fs->current, NULL, compress, 0, NULL);
+			if ( !fs->nffile ) {
+				LogError("Failed to open new collector file");
+				continue;
+			}
 		}
 
 		/* check for too little data - cnt must be > 0 at this point */
@@ -670,6 +690,7 @@ srecord_t	*commbuff;
 			continue;
 		}
 
+		fs->received = tv;
 		/* Process data - have a look at the common header */
 		version = ntohs(nf_header->version);
 		switch (version) {
@@ -743,7 +764,7 @@ int main(int argc, char **argv) {
  
 char	*bindhost, *filter, *datadir, pidstr[32], *launch_process;
 char	*userid, *groupid, *checkptr, *listenport, *mcastgroup, *extension_tags;
-char	*Ident, *pcap_file, pidfile[MAXPATHLEN];
+char	*Ident, *pcap_file, *dynsrcdir, pidfile[MAXPATHLEN];
 struct stat fstat;
 srecord_t	*commbuff;
 packet_function_t receive_packet;
@@ -783,8 +804,9 @@ int		c;
 	FlowSource		= NULL;
 	extension_tags	= DefaultExtensions;
 	pcap_file		= NULL;
+	dynsrcdir		= NULL;
 
-	while ((c = getopt(argc, argv, "46ef:whEVI:DB:b:j:l:n:p:P:R:S:s:T:t:x:Xru:g:z")) != EOF) {
+	while ((c = getopt(argc, argv, "46ef:whEVI:DB:b:j:l:M:n:p:P:R:S:s:T:t:x:Xru:g:z")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -829,6 +851,22 @@ int		c;
 			case 'I':
 				Ident = strdup(optarg);
 				break;
+			case 'M':
+				dynsrcdir = strdup(optarg);
+				if ( strlen(dynsrcdir) > MAXPATHLEN ) {
+					fprintf(stderr, "ERROR: Path too long!\n");
+					exit(255);
+				}
+				err  = stat(dynsrcdir, &fstat);
+				if ( !(fstat.st_mode & S_IFDIR) ) {
+					fprintf(stderr, "No such directory: %s\n", dynsrcdir);
+					break;
+				}
+				if ( !SetDynamicSourcesDir(&FlowSource, dynsrcdir) ) {
+					fprintf(stderr, "-l, -M and -n are mutually exclusive\n");
+					break;
+				}
+				break;
 			case 'n':
 				if ( AddFlowSource(&FlowSource, optarg) != 1 ) 
 					exit(255);
@@ -951,12 +989,14 @@ int		c;
 		}
 	}
 	
-	if ( FlowSource == NULL && datadir == NULL ) {
-		fprintf(stderr, "ERROR, Missing -n (-l/-I) source definitions\n");
+	if ( FlowSource == NULL && datadir == NULL && dynsrcdir == NULL ) {
+		fprintf(stderr, "ERROR, Missing -n (-l/-I) or -M source definitions\n");
 		exit(255);
 	}
-	if ( FlowSource == NULL && !AddDefaultFlowSource(&FlowSource, Ident, datadir) )
+	if ( FlowSource == NULL && datadir != NULL && !AddDefaultFlowSource(&FlowSource, Ident, datadir) ) {
+		fprintf(stderr, "Failed to add default data collector directory\n");
 		exit(255);
+	}
 
 	if ( bindhost && mcastgroup ) {
 		fprintf(stderr, "ERROR, -b and -j are mutually exclusive!!\n");
@@ -967,6 +1007,7 @@ int		c;
 		exit(255);
 	}
 
+	InitExtensionMaps(NULL);
 	SetupExtensionDescriptors(strdup(extension_tags));
 
 	// Debug code to read from pcap file
