@@ -58,6 +58,7 @@ typedef struct profile_channel_info_s {
 	char				*dirstat_path;	// pathname for dirstat file
 	nffile_t			*nffile;
 	stat_record_t		stat_record;
+	xstat_t				*xstat;
 	int					type;
 	dirstat_t 			*dirstat;
 } profile_channel_info_t;
@@ -65,7 +66,7 @@ typedef struct profile_channel_info_s {
 profile_channel_info_t	*GetProfiles(void);
 
 unsigned int InitChannels(char *profile_datadir, char *profile_statdir, profile_param_info_t *profile_list, 
-	char *filterfile, char *filename, int subdir_index, int veryfy_only, int compress );
+	char *filterfile, char *filename, int subdir_index, int veryfy_only, int compress, int do_xstat );
 
 profile_channel_info_t	*GetChannelInfoList(void);
 
